<?php

/*******************************************************************************/
//                          InstantCMS v1.10.4                                 //
//                      http://www.instantcms.ru/                              //                                                    //
//                  плагин "Статусы пользователей" v.1.1                       //
//                        (p_userstatuses)                                     //
//                     written by Marat Fatikhov                               //
//                      (nickname Марат on a site)                             //
//                       E-mail: f-marat@mail.ru                               //
//                                                                             //
//                      LICENSED BY GNU/GPL v2                                 //
//                                                                             //
/********************************************************************************/




class p_userstatuses extends cmsPlugin {

//=============================================================================//


public function __construct(){

        parent::__construct();

        // Информация о плагине

        $this->info['plugin']           = 'p_userstatuses';
        $this->info['title']            = 'User Statuses Plugin';
        $this->info['description']      = 'Добавляет вкладку "Статусы" в профили пользователей с возможностью просмотра предыдущих статусов, и комментарии к статусам';
        $this->info['author']           = 'Marat Fatikhov';
        $this->info['version']          = '1.1';

        $this->info['tab']              = 'Статусы'; //-- Заголовок закладки в профиле

        // Настройки по-умолчанию
        $this->config['Количество статусов во вкладке в профиле'] = 10;
        $this->config['Количество статусов на странице при просмотре всех статусов'] = 20;
        
        // События, которые будут отлавливаться плагином

        $this->events[]                 = 'USER_PROFILE';
        $this->events[]                 = 'AFTER_COMPONENT_USERS';
        $this->events[]                 = 'GET_ROUTE_USERS';
        $this->events[] = 'GET_USERS_ACTION_VIEWSTATUS';
        $this->events[] = 'GET_USERS_ACTION_VIEWSTATUSES';
        $this->events[] = 'GET_USERS_ACTION_DELSTATUS';
    }

// ==================================================================== //

    /**
     * Процедура установки плагина
     * @return bool
     */
    public function install(){

        $inDB = cmsDatabase::getInstance();

        //создаем таблицу для статусов в БД
        $table_sql = "CREATE TABLE IF NOT EXISTS cms_user_statuses(
                       id INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
                       user_id int(11) NOT NULL,
                       title varchar(255) NOT NULL,
                       status_date datetime NOT NULL)ENGINE=MyISAM  DEFAULT CHARSET=utf8;";
                       
        $inDB->query($table_sql);

        
        //добавляем цель для комментариев в БД
        if(!($inDB->get_field('`cms_comment_targets`', "target='statuses'", 'target'))){
         
         $target_sql = "INSERT INTO `cms_comment_targets` (target, component, title, target_table, subj) 
             VALUES ('statuses', 'users', 'Статусы пользователей', 'cms_user_statuses', 'вашего статуса')";
        
         $inDB->query($target_sql);
         }

        return parent::install();

    }

// ==================================================================== //

    /**
     * Процедура обновления плагина
     * @return bool
     */
    public function upgrade(){
        
        $inDB = cmsDatabase::getInstance();
        
        $sql = "DELETE FROM `cms_comment_targets` WHERE `target`='statuses'";
        $inDB->query($sql);
        $target_sql = "INSERT INTO `cms_comment_targets` (target, component, title, target_table, subj) 
             VALUES ('statuses', 'users', 'Статусы пользователей', 'cms_user_statuses', 'вашего статуса')";
        
         $inDB->query($target_sql);
         //изменяем название поля
         $change_sql = "ALTER TABLE `cms_user_statuses` 
             CHANGE `status_text` `title` VARCHAR( 255 ) 
             CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL ";
         $inDB->query($change_sql);
         
        return parent::upgrade();

    }

// ==================================================================== //

     /**
     * Обработка событий
     * @param string $event
     * @param mixed $item
     * @return mixed
     */
    public function execute($event, $item){

        parent::execute();

        switch ($event){
            case 'USER_PROFILE': $item = $this->eventUserProfile($item); break;
            case 'AFTER_COMPONENT_USERS': $item = $this->eventSetLink($item); break;
            case 'GET_ROUTE_USERS' : $item = $this->eventGetRoutes($item); break;
            case 'GET_USERS_ACTION_VIEWSTATUS' : $item = $this->getAction('viewstatus'); break;
            case 'GET_USERS_ACTION_VIEWSTATUSES' : $item = $this->getAction('viewstatuses'); break;
            case 'GET_USERS_ACTION_DELSTATUS' : $item = $this->getAction('delstatus'); break;
        }

        return $item;

    }
    
    // ==================================================================== //

    private function eventUserProfile($user){

        $inCore     = cmsCore::getInstance();
        $inUser     = cmsUser::getInstance();

        $statuses = array();

        $user_id    = $user['id'];
        if(!$user_id){ return 'Не получен идентификатор пользователя!'; }
        
        $limit      = $this->config['Количество статусов во вкладке в профиле'] ? $this->config['Количество статусов во вкладке в профиле'] : 10;

        $sql        = "SELECT id, title, status_date
                       FROM cms_user_statuses
                       WHERE user_id = '{$user_id}'
                       ORDER BY status_date DESC
                       LIMIT {$limit}";

        $result     = $this->inDB->query($sql);

        $total      = $this->inDB->num_rows($result);


        if ($total){

          while($status = $this->inDB->fetch_assoc($result)){
                $status['url'] = HOST.'/users/'.$user_id.'/status'.$status['id'].'.html' ;
                $status['del_url'] = HOST.'/users/'.$user_id.'/delstatus'.$status['id'].'.html' ;
                $status['comments_count'] = $inCore->getCommentsCount('statuses', $status['id']);
                $status['status_date'] = $inCore->dateFormat($status['status_date']);


                $statuses[] = $status;
            }


        }
        
        $myprofile = ($user_id == $inUser->id) ? true : false;

        ob_start();

        cmsPage::initTemplate('plugins', 'p_userstatusestab.tpl')->
        assign('total', $total)->
        assign('statuses', $statuses)->
        //если общ. количество статусов равно лимиту, считаем, что ещё есть статусы и показываем ссылку
        assign('all_statuses_link', $total == $limit)->
        assign('usr', $user)->
        assign('is_admin', $inUser->is_admin)->
        assign('myprofile', $myprofile)->        
        display('p_userstatusestab.tpl');

        $html = ob_get_clean();


          return  $html;
    }
    
    // ==================================================================== //
    
    private function eventSetLink($item){

    $inCore     = cmsCore::getInstance();
    $inDB   = cmsDatabase::getInstance();
    //$inUser   = cmsUser::getInstance();
    
    //получаем пользователя по текущему ури
    $login = trim(str_replace(cmsUSER::PROFILE_LINK_PREFIX, '',$_SERVER['REQUEST_URI']), '/');
    if(!$login){ return $item;}
    //получаем статус в таблице cms_users
    $r_status = $inDB->get_field('cms_users', "login='{$login}'", 'status');
    if(!$r_status){ return $item;}
    
    //статус в табл. cms_user_statuses
    $status = $inDB->get_fields('cms_user_statuses', "title = '{$r_status}'", 'id, user_id', 'id DESC');
    if(!$status['id']){ return $item;}
    
    //количество комментариев к статусу
    $count = $inCore->getCommentsCount('statuses', $status['id']);
    
    $source = '/(.*?)<span class="usr_status_date" >(.*?)<\\/span>(.*?)/ui';
    
    //в переменной ниже устанавливается ссылка "Комментировать". Можете изменить, как нужно.
    $link = '<a style="font-size:13px;" href="/users/'.$status['user_id'].'/status'.$status['id'].'.html">('.$count.') Комментировать</a>';
    
    $replacement = '$1<span class="usr_status_date" >$2 &nbsp;&nbsp;&nbsp;'.$link.' </span>$3';
    $item = preg_replace($source, $replacement, $item);

    return $item;
    }
    
    // ==================================================================== //

    private function eventGetRoutes($routes) {
		
		// формируем массив по аналогии с router.php
		
         $add_routes[] = array(
                            '_uri'  => '/^users\/([0-9]+)\/status([0-9]+).html$/i',
                            'do'    => 'viewstatus',
                            1       => 'id',
                            2       => 'statusid'
                         );
                         
          $add_routes[]=array(
                            '_uri'  => '/^users\/([0-9]+)\/statuses.html$/i',
                            'do'    => 'viewstatuses',
                            1       => 'id'
                           );
          
          $add_routes[]=array(
                            '_uri'  => '/^users\/([0-9]+)\/statuses-([0-9]+).html$/i',
                            'do'    => 'viewstatuses',
                            1       => 'id',
                            2       => 'page'
                           );
        
           $add_routes[] = array(
                            '_uri'  => '/^users\/([0-9]+)\/delstatus([0-9]+).html$/i',
                            'do'    => 'delstatus',
                            1       => 'id',
                            2       => 'statusid'
                            );

		// перебираем массив $add_routes, занося каждый в начало входного массива $routes
		foreach($add_routes as $route){
			array_unshift($routes, $route);
		}
                
        return $routes;

    }

// ==================================================================== //
    private function getAction($action){
        
        if(!$action) return false;
            
       $file = PATH.'/components/users/plugins/'.$action.'.php';
       
       include $file;
        
        return true;
    }
// ==================================================================== //    
}
?>
